"""
Django-tagging
"""
__version__ = '0.5.3'
__license__ = 'BSD License'

__author__ = 'Jonathan Buchanan'
__author_email__ = 'jonathan.buchanan@gmail.com'

__maintainer__ = 'Arturo Inzunza'
__maintainer_email__ = 'arturo@plateiq.com'

__url__ = 'https://github.com/inzunzo/django-tagging-qubiqle'

default_app_config = 'tagging.apps.TaggingConfig'
